from abc import ABC, abstractmethod
from typing import List, Optional
import uuid

from .types import AgentCard
from .query_parser import QueryParser
from .log import get_logger

log = get_logger(__name__)


class AgentRegistry(ABC):
    @abstractmethod
    async def add(self, agent: AgentCard) -> AgentCard:
        pass

    @abstractmethod
    async def get(self, agent_id: str) -> Optional[AgentCard]:
        pass

    @abstractmethod
    async def delete(self, agent_id: str) -> None:
        pass

    @abstractmethod
    async def list(self) -> List[AgentCard]:
        pass

    @abstractmethod
    async def search(self, query: str) -> List[AgentCard]:
        pass

    @abstractmethod
    async def update(self, agent_id: str, agent: AgentCard) -> AgentCard:
        pass

    @abstractmethod
    async def clear(self) -> None:
        pass

    @abstractmethod
    async def close(self) -> None:
        pass


class InMemoryRegistry(AgentRegistry):
    def __init__(self, query_parser: Optional[QueryParser] = None):
        self._agents: dict[str, AgentCard] = {}
        self._query_parser = query_parser or QueryParser()
        log.info("In-memory registry initialized")

    async def add(self, agent: AgentCard) -> AgentCard:
        # Ensure name is unique
        for existing_agent in self._agents.values():
            if existing_agent.name == agent.name:
                raise ValueError(f"Agent with name {agent.name} already exists.")

        if not agent.id:
            agent.id = str(uuid.uuid4())
        log.info(f"Adding agent {agent.name} ({agent.id}) to in-memory registry")
        self._agents[agent.id] = agent
        return agent

    async def get(self, agent_id: str) -> Optional[AgentCard]:
        log.info(f"Getting agent {agent_id} from in-memory registry")
        return self._agents.get(agent_id)

    async def delete(self, agent_id: str) -> None:
        log.info(f"Removing agent {agent_id} from in-memory registry")
        if agent_id in self._agents:
            del self._agents[agent_id]

    async def list(self) -> List[AgentCard]:
        log.info("Listing all agents in in-memory registry")
        return list(self._agents.values())

    async def update(self, agent_id: str, agent_update: AgentCard) -> AgentCard:
        log.info(f"Updating agent {agent_id} in in-memory registry")
        if agent_id not in self._agents:
            raise ValueError(f"Agent with ID {agent_id} not found.")
        self._agents[agent_id] = agent_update
        return agent_update

    async def search(self, query: str) -> List[AgentCard]:
        log.info(f"Searching for '{query}' in in-memory registry")
        agents = list(self._agents.values())

        if not query or not query.strip():
            return agents

        parsed_query = self._query_parser.parse(query)

        if not parsed_query.field_filters and not parsed_query.general_filters:
            return agents

        # Apply field filters
        for f in parsed_query.field_filters:
            if f.operator == 'has_skill':
                agents = [
                    agent for agent in agents
                    if any(
                        s.id.lower() == f.value.lower() or s.name.lower() == f.value.lower()
                        for s in agent.skills
                    )
                ]
            else:
                agents = [
                    agent for agent in agents
                    if hasattr(agent, f.field) and f.value.lower() in str(getattr(agent, f.field)).lower()
                ]

        # Apply general text search filters
        for f in parsed_query.general_filters:
            agents = [
                agent for agent in agents
                if any(
                    hasattr(agent, field) and f.term.lower() in str(getattr(agent, field)).lower()
                    for field in f.fields
                )
            ]
        
        log.info(f"Search for '{query}' returned {len(agents)} results")
        return agents

    async def clear(self) -> None:
        log.info("Clearing all agents from in-memory registry")
        self._agents.clear()

    async def close(self) -> None:
        log.info("Closing in-memory registry")
        self._agents.clear()



import logging
import os
import colorlog


def setup_logger():
    """
    Set up the logger for the OpenHive SDK.
    """
    log_level_str = os.getenv("LOG_LEVEL", "INFO").upper()
    log_level = getattr(logging, log_level_str, logging.INFO)

    logger = logging.getLogger("openhive")
    logger.setLevel(log_level)

    # Prevent duplicate handlers if this function is called multiple times
    if logger.hasHandlers():
        logger.handlers.clear()

    handler = colorlog.StreamHandler()
    formatter = colorlog.ColoredFormatter(
        '%(log_color)s%(levelname)-8s%(reset)s %(name)s: %(message)s',
        log_colors={
            'DEBUG': 'cyan',
            'INFO': 'green',
            'WARNING': 'yellow',
            'ERROR': 'red',
            'CRITICAL': 'bold_red',
        }
    )
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    return logger


log = setup_logger()


def get_logger(name: str):
    """
    Get a namespaced logger.
    """
    return logging.getLogger(f"openhive.{name}")



import sqlite3
import json
import uuid
from typing import List, Optional

from .agent_registry import AgentRegistry
from .types import AgentCard, Skill
from .log import get_logger
from .query_parser import QueryParser


log = get_logger(__name__)


class SqliteRegistry(AgentRegistry):
    def __init__(self, db_path: str, query_parser: Optional[QueryParser] = None):
        self._db_path = db_path
        self._conn = sqlite3.connect(db_path)
        self._conn.row_factory = sqlite3.Row
        self._query_parser = query_parser or QueryParser()
        self._create_table()
        log.info(f"SQLite registry initialized at {db_path}")

    def _create_table(self):
        cursor = self._conn.cursor()
        cursor.execute('''
            CREATE TABLE IF NOT EXISTS agents (
                id TEXT PRIMARY KEY,
                name TEXT,
                description TEXT,
                protocolVersion TEXT,
                version TEXT,
                url TEXT,
                skills TEXT
            )
        ''')
        cursor.execute('CREATE UNIQUE INDEX IF NOT EXISTS idx_agent_name ON agents (name)')
        self._conn.commit()

    async def add(self, agent: AgentCard) -> AgentCard:
        if not agent.id:
            agent.id = str(uuid.uuid4())
        log.info(f"Adding agent {agent.name} ({agent.id}) to SQLite registry")
        cursor = self._conn.cursor()
        try:
            cursor.execute(
                'INSERT INTO agents (id, name, description, protocolVersion, version, url, skills) VALUES (?, ?, ?, ?, ?, ?, ?)',
                (
                    agent.id,
                    agent.name,
                    agent.description,
                    agent.protocol_version,
                    agent.version,
                    agent.url,
                    json.dumps([s.dict() for s in agent.skills]),
                ),
            )
            self._conn.commit()
        except sqlite3.IntegrityError:
            raise ValueError(f"Agent with name {agent.name} already exists.")
        return agent

    async def get(self, agent_id: str) -> Optional[AgentCard]:
        log.info(f"Getting agent {agent_id} from SQLite registry")
        cursor = self._conn.cursor()
        cursor.execute('SELECT * FROM agents WHERE id = ?', (agent_id,))
        row = cursor.fetchone()
        if not row:
            return None
        return self._row_to_agent(row)

    async def delete(self, agent_id: str) -> None:
        log.info(f"Removing agent {agent_id} from SQLite registry")
        cursor = self._conn.cursor()
        cursor.execute('DELETE FROM agents WHERE id = ?', (agent_id,))
        self._conn.commit()

    async def list(self) -> List[AgentCard]:
        log.info(f"Listing all agents from SQLite registry")
        cursor = self._conn.cursor()
        cursor.execute('SELECT * FROM agents')
        rows = cursor.fetchall()
        return [self._row_to_agent(row) for row in rows]

    async def update(self, agent_id: str, agent: AgentCard) -> AgentCard:
        log.info(f"Updating agent {agent_id} in SQLite registry")
        cursor = self._conn.cursor()
        cursor.execute(
            '''
            UPDATE agents
            SET name = ?, description = ?, protocolVersion = ?, version = ?, url = ?, skills = ?
            WHERE id = ?
            ''',
            (
                agent.name,
                agent.description,
                agent.protocol_version,
                agent.version,
                agent.url,
                json.dumps([s.dict() for s in agent.skills]),
                agent_id,
            ),
        )
        self._conn.commit()
        return agent
        
    async def search(self, query: str) -> List[AgentCard]:
        log.info(f"Searching for '{query}' in SQLite registry")
        agents = await self.list()

        if not query or not query.strip():
            return agents

        parsed_query = self._query_parser.parse(query)

        if not parsed_query.field_filters and not parsed_query.general_filters:
            return agents

        # Apply field filters
        for f in parsed_query.field_filters:
            if f.operator == 'has_skill':
                agents = [
                    agent for agent in agents
                    if any(
                        s.id.lower() == f.value.lower() or s.name.lower() == f.value.lower()
                        for s in agent.skills
                    )
                ]
            else:
                agents = [
                    agent for agent in agents
                    if hasattr(agent, f.field) and f.value.lower() in str(getattr(agent, f.field)).lower()
                ]

        # Apply general text search filters
        for f in parsed_query.general_filters:
            agents = [
                agent for agent in agents
                if any(
                    hasattr(agent, field) and f.term.lower() in str(getattr(agent, field)).lower()
                    for field in f.fields
                )
            ]
            
        return agents

    async def clear(self) -> None:
        log.info("Clearing all agents from SQLite registry")
        cursor = self._conn.cursor()
        cursor.execute('DELETE FROM agents')
        self._conn.commit()

    async def close(self) -> None:
        log.info("Closing SQLite registry connection")
        self._conn.close()

    def _row_to_agent(self, row: sqlite3.Row) -> AgentCard:
        return AgentCard(
            id=row['id'],
            name=row['name'],
            description=row['description'],
            protocolVersion=row['protocolVersion'],
            version=row['version'],
            url=row['url'],
            skills=[Skill(**s) for s in json.loads(row['skills'])],
        )



from typing import List, Optional

from .agent_registry import AgentRegistry, InMemoryRegistry
from .remote_registry import RemoteRegistry
from .types import AgentCard
from .query_parser import QueryParser


class OpenHive:
    def __init__(
        self,
        registry_url: Optional[str] = None,
        auth_token: Optional[str] = None,
        query_parser: Optional[QueryParser] = None,
        registry: Optional[AgentRegistry] = None,
    ):
        if registry:
            self._registry = registry
        elif registry_url:
            self._registry = RemoteRegistry(endpoint=registry_url, token=auth_token)
        else:
            self._registry = InMemoryRegistry(query_parser=query_parser)

    async def add(self, agent: AgentCard) -> AgentCard:
        return await self._registry.add(agent)

    async def get(self, agent_id: str) -> Optional[AgentCard]:
        return await self._registry.get(agent_id)

    async def list(self) -> List[AgentCard]:
        return await self._registry.list()

    async def update(self, agent_id: str, agent: AgentCard) -> AgentCard:
        return await self._registry.update(agent_id, agent)

    async def delete(self, agent_id: str) -> None:
        await self._registry.delete(agent_id)

    async def search(self, query: str) -> List[AgentCard]:
        return await self._registry.search(query)

    async def close(self) -> None:
        await self._registry.close()



"""
OpenHive Core SDK for Python
"""
__version__ = "0.11.0"

from .log import get_logger
from .query_parser import QueryParser
from .agent_registry import AgentRegistry, InMemoryRegistry
from .remote_registry import RemoteRegistry
from .sqlite_registry import SqliteRegistry
from .types import AgentCard, Skill
from .core import OpenHive

__all__ = [
    "get_logger",
    "QueryParser",
    "AgentRegistry",
    "InMemoryRegistry",
    "RemoteRegistry",
    "SqliteRegistry",
    "AgentCard",
    "Skill",
    "OpenHive",
]



import httpx
from typing import List, Optional

from .agent_registry import AgentRegistry
from .types import AgentCard
from .log import get_logger

log = get_logger(__name__)


class RemoteRegistry(AgentRegistry):
    def __init__(self, endpoint: str, token: Optional[str] = None):
        self._endpoint = endpoint
        self._token = token
        log.info(f"Remote registry adapter initialized for endpoint: {endpoint}")

    @property
    def _headers(self) -> dict:
        headers = {"Content-Type": "application/json"}
        if self._token:
            headers["Authorization"] = f"Bearer {self._token}"
        return headers

    async def add(self, agent: AgentCard) -> AgentCard:
        agent_id = agent.name
        log.info(f"Adding agent {agent_id} to remote registry")
        async with httpx.AsyncClient() as client:
            response = await client.post(
                f"{self._endpoint}/agents",
                content=agent.json(by_alias=True),
                headers=self._headers,
            )
            response.raise_for_status()
            return AgentCard(**response.json())

    async def get(self, agent_id: str) -> Optional[AgentCard]:
        log.info(f"Getting agent {agent_id} from remote registry")
        async with httpx.AsyncClient() as client:
            response = await client.get(
                f"{self._endpoint}/agents/{agent_id}", headers=self._headers
            )
            if response.status_code == 404:
                return None
            response.raise_for_status()
            return AgentCard(**response.json())

    async def delete(self, agent_id: str) -> None:
        log.info(f"Removing agent {agent_id} from remote registry")
        async with httpx.AsyncClient() as client:
            response = await client.delete(
                f"{self._endpoint}/agents/{agent_id}", headers=self._headers
            )
            response.raise_for_status()

    async def list(self) -> List[AgentCard]:
        log.info(f"Listing agents from remote registry")
        async with httpx.AsyncClient() as client:
            response = await client.get(f"{self._endpoint}/agents", headers=self._headers)
            response.raise_for_status()
            return [AgentCard(**info) for info in response.json()]

    async def search(self, query: str) -> List[AgentCard]:
        log.info(f"Searching for '{query}' in remote registry")
        async with httpx.AsyncClient() as client:
            response = await client.get(
                f"{self._endpoint}/agents",
                params={'q': query},
                headers=self._headers,
            )
            response.raise_for_status()
            return [AgentCard(**info) for info in response.json()]

    async def update(self, agent_id: str, agent: AgentCard) -> AgentCard:
        log.info(f"Updating agent {agent_id} in remote registry")
        async with httpx.AsyncClient() as client:
            response = await client.put(
                f"{self._endpoint}/agents/{agent_id}",
                content=agent.json(by_alias=True),
                headers=self._headers,
            )
            response.raise_for_status()
            return AgentCard(**response.json())

    async def clear(self) -> None:
        log.warning("Clear operation is not supported on a remote registry.")
        raise NotImplementedError("Clear operation is not supported on a remote registry.")

    async def close(self) -> None:
        log.info("No-op for remote registry close")
        pass



from pydantic import BaseModel, Field
from typing import List, Optional, Dict, Any


class Skill(BaseModel):
    id: str
    name: str
    description: Optional[str] = None
    tags: Optional[List[str]] = []
    input: Optional[Dict[str, Any]] = {}
    output: Optional[Dict[str, Any]] = {}


class AgentCard(BaseModel):
    id: Optional[str] = None  # Optional on creation, will be assigned by the registry
    name: str
    description: Optional[str] = None
    protocol_version: str = Field(..., alias='protocolVersion')
    version: str
    url: str
    skills: List[Skill]
    capabilities: Optional[Dict[str, bool]] = None



import re
from typing import Literal, List


class QueryFilter:
    def __init__(self, field: str, value: str, operator: Literal['includes', 'equals', 'has_skill']):
        self.field = field
        self.value = value
        self.operator = operator


class GeneralFilter:
    def __init__(self, term: str, fields: List[str]):
        self.term = term
        self.fields = fields


class ParsedQuery:
    def __init__(self):
        self.field_filters: List[QueryFilter] = []
        self.general_filters: List[GeneralFilter] = []


class QueryParser:
    def parse(self, query: str) -> ParsedQuery:
        result = ParsedQuery()

        if not query or not query.strip():
            return result

        parts = re.findall(r'(?:[^\s"]+|"[^"]*")+', query)

        for part in parts:
            value = part
            if value.startswith('"') and value.endswith('"'):
                value = value[1:-1]

            if ':' in part:
                field, *value_parts = part.split(':', 1)
                field_value = ':'.join(value_parts)
                if field_value.startswith('"') and field_value.endswith('"'):
                    field_value = field_value[1:-1]

                if field and field_value:
                    if field.lower() == 'skill':
                        result.field_filters.append(QueryFilter('skills', field_value, 'has_skill'))
                    else:
                        result.field_filters.append(QueryFilter(field, field_value, 'includes'))
            else:
                result.general_filters.append(GeneralFilter(value, ['name', 'description']))
        
        return result